string Description = "Basic Bump";

texture NormalTexture : Normal
<
    string ResourceName = "normal.dds";
>;

sampler NormalMap = sampler_state
{
    texture = <NormalTexture>;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

texture EnviTexture : Environment
<
    string ResourceName = "cube.dds";
    string TextureType = "Cube";
>;

sampler EnviMap = sampler_state
{
    texture = <EnviTexture>;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

float4x4 WVP : WorldViewProjection;
float4x4 W   : World;
float4x4 WIT : WorldInverseTranspose;
float4x4 VI  : ViewInverse;

struct AppData
{
    float3 P  : POSITION;
    float3 N  : NORMAL;
    float3 T  : TANGENT;
    float3 B  : BINORMAL;
    float2 UV : TEXCOORD0;
};

struct VertexOutput
{
    float4 HP : POSITION;
    float2 UV : TEXCOORD0;
    float3 N  : TEXCOORD1;
    float3 T  : TEXCOORD2;
    float3 B  : TEXCOORD3;
    float3 V  : TEXCOORD4;
};

struct PixelOutput
{    
    float4 color : COLOR;
};

VertexOutput VS_Bump(AppData IN)
{
    VertexOutput OUT;
    float4 PObject = float4(IN.P, 1.0);
    OUT.HP = mul(PObject, WVP);
    OUT.N = normalize(mul(float4(IN.N, 0.0), WIT).xyz);
    OUT.T = normalize(mul(float4(IN.T, 0.0), W).xyz);
    OUT.B = normalize(mul(float4(IN.B, 0.0), W).xyz);
    float4 PWorld = mul(PObject, W);
    OUT.V = normalize(VI[3].xyz - PWorld.xyz);
    OUT.UV = IN.UV;
    return OUT;
}

PixelOutput PS_Bump(VertexOutput IN)
{
    PixelOutput OUT;
    float3 N = normalize(IN.N);
    float3 T = normalize(IN.T);
    float3 B = normalize(IN.B);
    float3 NObj = 2.0*tex2D(NormalMap, IN.UV).xyz - 1.0;
    float3 NWorld = normalize(NObj.x*T + NObj.y*B + NObj.z*N);
    OUT.color = texCUBE(EnviMap, reflect(-IN.V, NWorld));
    return OUT;
}

technique simpleTechnique
{
    pass p0 
    {        
        VertexShader = compile vs_2_0 VS_Bump();
        PixelShader  = compile ps_2_0 PS_Bump();
    }
}